#
# tcllauncher.tcl - tcl code that tcllauncher uses to do its thing
#

package require Tclx

#
# this is the code that gets called when Tcl launches because the
# tcllauncher "version" of Tcl jimmies the command line to make it
# so.
#
# this routine must now figure out what to do to launch the app
# 
#
proc main {{argv ""}} {
    set prog [info nameofexecutable]

    # have we been invoked as a shell?  If so, prog is empty, get it from
    # the SHELL environment variable
    if {$prog == ""} {
	set prog $::env(SHELL)
    }

    set path [file split $prog]
    set shortName [lindex $path end]

    #
    # tcllauncher cannot be invoked directly as "tcllauncher" -- it must be
    # aliased to some other name
    #
    if {$shortName == "tcllauncher"} {
        puts stderr "tcllauncher cannot be invoked as \"tcllauncher\"; it must be copied or linked as some other name"
	exit 255
    }

    # if the last dir in the chain is "bin", swap in "lib/tcllauncher" in
    # its place and tag a ".tcl" onto the end of the path name.
    #
    # otherwise just look in the same directory where the instance of the
    # tcllauncher was found.
    #

    #puts stderr "path '$path', prog '$prog', shortName '$shortName'"

    if {[lindex $path end-1] == "bin"} {
	# This looks for
	# /usr/share/tcl<version>/$shortName/main.tcl
	# /usr/lib64/tcl<version>/$shortName/main.tcl (on 64bit platforms)
	# /usr/lib/tcl<version>/$shortName/main.tcl (on 32bit platforms)
	#
	# These are the paths that Fedora uses for Tcl packages
	if {$::tcl_platform(pointerSize) == 8} {
	    set libdir "lib64"
	} else {
	    set libdir "lib"
	}
	set tcldir tcl$::tcl_version
	set paths [list /usr/share/$tcldir /usr/$libdir/$tcldir "" ]
	foreach candidate $paths {
	    if {$candidate == ""} {
		puts stderr "$shortName: can't find in paths (tcllauncher)"
		exit 254
	    }
	    set ::launchdir [eval file join [list $candidate $shortName]]
	    set path [eval file join $::launchdir main.tcl]
	    if {[file readable $path]} {
		break
	    }
	}
    } else {
        set path $prog.tcl
    }

    set ::argv0 $shortName
    set initialArgv $argv

    # ok now source in the file we (tcllauncher) figured out is the one

    if {[catch {uplevel #0 source $path} catchResult] == 1} {
        append ::errorInfo "\n    from tcllauncher running \"[string trim "$::argv0 $initialArgv"]\""
	puts stderr $::errorInfo
        exit 255
    }

    exit 0
}

if {!$tcl_interactive} {
    main $argv
}

# vim: set ts=8 sw=4 sts=4 noet :
